@@ -613,7 +613,11 @@ void KateRenderer::assignSelectionBrushesFromAttribute(QTextLayout::FormatRange
     }
 }
 
-void KateRenderer::paintTextBackground(QPainter &paint, KateLineLayout *layout, const QList<QTextLayout::FormatRange> &selRanges, const QBrush &brush) const
+void KateRenderer::paintTextBackground(QPainter &paint,
+                                       KateLineLayout *layout,
+                                       const QList<QTextLayout::FormatRange> &selRanges,
+                                       const QBrush &brush,
+                                       int xStart) const
 {
     const bool rtl = layout->isRightToLeft();
 
@@ -637,8 +641,11 @@ void KateRenderer::paintTextBackground(QPainter &paint, KateLineLayout *layout,
         const int endViewLine = layout->viewLineForColumn(e);
         if (startViewLine == endViewLine) {
             KateTextLayout l = layout->viewLine(startViewLine);
-            const int startX = cursorToX(l, s);
-            const int endX = cursorToX(l, e);
+            // subtract xStart so that the selection is shown where it belongs
+            // we don't do it in the else case because this only matters when dynWrap==false
+            // and when dynWrap==false, we always have 1 QTextLine per layout
+            const int startX = cursorToX(l, s) - xStart;
+            const int endX = cursorToX(l, e) - xStart;
             const int y = startViewLine * lineHeight();
             QRect r(startX, y, (endX - startX), lineHeight());
             paint.fillRect(r, br);
@@ -756,15 +763,16 @@ void KateRenderer::paintTextLine(QPainter &paint,
                                                return sr.overlapsLine(line) && sr.overlapsColumn(fr.start) && fr.format.background().color() == c;
                                            }),
                             decos.end());
-                paintTextBackground(paint, range, decos, Qt::NoBrush);
+                paintTextBackground(paint, range, decos, Qt::NoBrush, xStart);
             }
 
             if (drawSelection) {
                 additionalFormats = decorationsForLine(range->textLine(), range->line(), true);
                 if (hasCustomLineHeight()) {
-                    paintTextBackground(paint, range, additionalFormats, config()->selectionColor());
+                    paintTextBackground(paint, range, additionalFormats, config()->selectionColor(), xStart);
                 }
-                range->layout()->draw(&paint, QPoint(-xStart, 0), additionalFormats, textClipRect);
+                // DONT apply clipping, it breaks rendering when there are selections
+                range->layout()->draw(&paint, QPoint(-xStart, 0), additionalFormats);
 
             } else {
                 range->layout()->draw(&paint, QPoint(-xStart, 0), QList<QTextLayout::FormatRange>{}, textClipRect);
@@ -904,7 +912,7 @@ void KateRenderer::paintTextLine(QPainter &paint,
                 const int y = lineHeight() * i + m_fontAscent;
 
                 static const QRegularExpression nonPrintableSpacesRegExp(
-                    QStringLiteral("[\\x{2000}-\\x{200F}\\x{2028}-\\x{202F}\\x{205F}-\\x{2064}\\x{206A}-\\x{206F}]"));
+                    QStringLiteral("[\\x{0000}-\\x{0008}\\x{000A}-\\x{001F}\\x{2000}-\\x{200F}\\x{2028}-\\x{202F}\\x{205F}-\\x{2064}\\x{206A}-\\x{206F}]"));
                 QRegularExpressionMatchIterator i = nonPrintableSpacesRegExp.globalMatch(text, line.lineLayout().xToCursor(xStart));
 
                 while (i.hasNext()) {
@@ -1100,11 +1108,13 @@ void KateRenderer::paintCaret(KTextEditor::Cursor cursor, KateLineLayout *range,
                 xStart += caretWidth;
             }
             qreal width = 0;
-            const auto inlineNotes = m_view->inlineNotes(range->line());
-            for (const auto &inlineNoteData : inlineNotes) {
-                KTextEditor::InlineNote inlineNote(inlineNoteData);
-                if (inlineNote.position().column() == cursor.column()) {
-                    width = inlineNote.width() + (caretStyle() == KTextEditor::caretStyles::Line ? 2.0 : 0.0);
+            if (cursor.column() < range->length()) {
+                const auto inlineNotes = m_view->inlineNotes(range->line());
+                for (const auto &inlineNoteData : inlineNotes) {
+                    KTextEditor::InlineNote inlineNote(inlineNoteData);
+                    if (inlineNote.position().column() == cursor.column()) {
+                        width = inlineNote.width() + (caretStyle() == KTextEditor::caretStyles::Line ? 2.0 : 0.0);
+                    }
                 }
             }
             drawCursor(*range->layout(), &paint, QPoint(-xStart - width, 0), cursor.column(), caretWidth, lineHeight());
@@ -1339,7 +1349,7 @@ void KateRenderer::layoutLine(KateLineLayout *lineLayout, int maxwidth, bool cac
                 const qreal caretWidth = caretStyle() == KTextEditor::caretStyles::Line ? 2.0 : 0.0;
                 text_char_format.setFontLetterSpacing(width + caretWidth);
                 text_char_format.setFontLetterSpacingType(QFont::AbsoluteSpacing);
-                decorations.append(QTextLayout::FormatRange{column - 1, 1, text_char_format});
+                decorations.append(QTextLayout::FormatRange{.start=column - 1, .length=1, .format=text_char_format});
             }
         }
     }
@@ -1481,7 +1491,7 @@ void KateRenderer::setCaretOverrideColor(const QColor &color)
     m_caretOverrideColor = color;
 }
 
-void KateRenderer::paintSelection(QPaintDevice *d, int startLine, int xStart, int endLine, int xEnd, qreal scale)
+void KateRenderer::paintSelection(QPaintDevice *d, int startLine, int xStart, int endLine, int xEnd, int viewWidth, qreal scale)
 {
     if (!d || scale < 0.0) {
         return;
@@ -1513,13 +1523,13 @@ void KateRenderer::paintSelection(QPaintDevice *d, int startLine, int xStart, in
         // compute layout WITHOUT cache to not poison it + render it
         KateLineLayout lineLayout(*this);
         lineLayout.setLine(line, -1);
-        layoutLine(&lineLayout, -1 /* no wrap */, false /* no layout cache */);
+        layoutLine(&lineLayout, viewWidth, false /* no layout cache */);
         KateRenderer::PaintTextLineFlags flags;
         flags.setFlag(KateRenderer::SkipDrawFirstInvisibleLineUnderlined);
         flags.setFlag(KateRenderer::SkipDrawLineSelection);
         paintTextLine(paint, &lineLayout, 0, 0, QRectF{}, nullptr, flags);
 
         // translate for next line
-        paint.translate(0, lineHeight);
+        paint.translate(0, lineHeight * lineLayout.viewLineCount());
     }
 }
